@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2014 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -83,7 +83,7 @@ cao  <- function(formula,
   cao.fitter <- get(method)
 
 
-  deviance.Bestof <- rep(as.numeric(NA), len = control$Bestof)
+  deviance.Bestof <- rep(NA_real_, len = control$Bestof)
   for (tries in 1:control$Bestof) {
     if (control$trace && (control$Bestof > 1)) {
       cat(paste("\n========================= Fitting model",
